#!/usr/bin/env python

# Copyright (C) 2007 CEREA
#   Author: Meryem Ahmed de Biasi
#
# This code is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This file is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# This module provides a graphical user interface for Polyphemus.

import matplotlib
matplotlib.rcParams["backend"] = "Agg"
matplotlib.rcParams["interactive"] = False 
matplotlib.rcParams["figure.subplot.left"] = 0.05
matplotlib.rcParams["figure.subplot.right"] = 0.95
matplotlib.rcParams["figure.subplot.bottom"] = 0
matplotlib.rcParams["figure.subplot.top"] = 1


import os
from matplotlib.toolkits.basemap import Basemap
from matplotlib.backends.backend_qtagg import FigureCanvasQTAgg as FigureCanvas
from qt import *
from qtcanvas import *
from simulation import *
from atmopy import *
from numpy import *
from pylab import *

# Various functions to manage dates.
def split_date(Date):
	"""
	Convert a date.

	@type Date: String.
	@param Date: The Date to convert.

	@rtype: List.
	@return: A list giving the year, month, day, hour, minute and
	seconds.
	"""
	Date_tmp =  "".join(Date.split("-"))
	Date_tmp =  "".join(Date_tmp.split("_"))
	year = int(Date_tmp[0:4])
	month = int(Date_tmp[4:6])
	day = int(Date_tmp[6:8])
	hour = int(Date_tmp[8:10])
	minute = int(Date_tmp[10:12])
	second = int(Date_tmp[12:14])
	Date_res = [year, month, day, hour, minute, second]
	return Date_res

def is_leap_year(year):
	"""
	Check wheter a given year is a leap year.

	@type year: Integer.
	@param year: The year to check.

	@rtype: Boolean.
	@return: True if the year is a leap year, False otherwise.
	"""
	return year % 4 == 0 and (year % 100 != 0 or year % 400 == 0)
	
def number_days(year, month):
	"""
	Returns the number of days of a given month.

	@type year: Integer.
	@param year: The year to check.

	@type month: Integer.
	@param month: The month to check.

	@rtype: Integer.
	@return: The number of days in the month considered.
	"""
	if month in [1, 3, 5, 7, 8, 10, 12]:
		n = 31
	elif month in [4, 6, 9]:
		n =  30
	else:
		if is_leap_year(year):
			n = 29
		else:
			n = 28
	return n
		
def get_seconds(year, month, day, hour, minute, second):
	"""
	Computes the number of seconds from the beginning of the year.

	@type year: Integer.
	@param year: The year considered.

	@type month: Integer.
	@param month: The month considered.

	@type day: Integer.
	@param day: The day considered.

	@type hour: Integer.
	@param hour: The hour considered.

	@type minute: Integer.
	@param minute: The minute considered.

	@type second: Integer.
	@param second: The second considered.

	@rtype: Integer.
	@return: The number of seconds since the beginning of year.	
	"""
	# Number of days from the beginning of the year to beginning of month.
	day_month = 0
	for m in range(1, month):
		day_month += number_days(year, m)
	seconds = second + 60 * (minute + 60 * (hour + 24 * (day + day_month)))
	return seconds

def compute_Nt(Date1, Date2, Delta):
	"""
	Compute the number of iterations between Date1 and Date2 (excluded)
with a time step of Delta seconds.

	@type Date1: String.
	@param Date1: First date considered.

	@type Date2: String.
	@param Date2: Second date considered.

	@type Delta: Integer.
	@param Delta: Time step in seconds.

	@rtype: Integer.
	@return: The number of iterations to perform.
	"""
	l1 = split_date(Date1)
	sec1 = get_seconds(l1[0], l1[1], l1[2], l1[3], l1[4], l1[5])
	l2 = split_date(Date2)
	sec2 = get_seconds(l2[0], l2[1], l2[2], l2[3], l2[4], l2[5])
	if l1[0] == l2[0]:
		seconds = sec2 - sec1
	else:
		days = 0
		for y in range(year1, year2):
			if is_leap_year(y):
				days += 366
			else:
				days += 365
		seconds = days * 24 * 3600 + sec2 - sec1
	Nt = seconds / Delta
	return Nt

def AddHours(Date, hours):
	"""
	Add hours to a date. The number of hours can be negative.

	@type Date: String.
	@param Date: Date considered.

	@type hours: Integer.
	@param hours: The number of hours to add to the date.

	@rtype: String.
	@return: The new date. 
	"""
	l= split_date(Date)
	l[3] += hours
	if l[3] > 23:
		l[2] += l[3] / 24
		l[3] = l[3] % 24
	elif l[3] < 0:
		l[2] -= abs(l[3]) / 24 + 1
		l[3] = 24 - abs(l[3])  % 24

	if l[2] > number_days(l[0], l[1]):
		while l[2] > number_days(l[0], l[1]):
			l[2] -= number_days(l[0], l[1])
			l[1] += 1
	elif l[2] < 1:
		while l[2] < 1:
			l[2] += number_days(l[0], l[1])
			l[1] -= 1

	if l[1] > 12:
		l[0] += l[1] / 12
		l[1] = l[1] % 12
	elif l[0] < 1:
		l[0] = l[0] - l[1] / 12 - 1
		l[1] = 12 - l[1] % 12

	Date_tmp = str(l[0])
	if l[1] < 10:
		Date_tmp += "-0" +str(l[1])
	else:
		Date_tmp += "-" +str(l[1])
	if l[2] < 10:
		Date_tmp += "-0" +str(l[2])
	else:
		Date_tmp += "-" +str(l[2])
	if l[3] < 10:
		Date_tmp += "_0" +str(l[3])
	else:
		Date_tmp += "_" +str(l[3])
	if l[4] <10:
		Date_tmp += "-0" +str(l[4])
	else:
		Date_tmp += "-" +str(l[4])
	if l[5] <10:	
		Date_tmp += "-0" +str(l[5])
	else:
		Date_tmp += "-" +str(l[5])
	return Date_tmp


# Definition of several classes.

class ConfigItem:
	"""
	Defines items of a configuration file.
	Each item has three attributes: a name, a section and a value.  This
	way one can modify the value of an item and still access the item
	through its name.
	"""
	def __init__(self, name, section, value):
		self.name = name
		self.section = section
		self.value = value

class PolyphemusConfig:
	"""
	Stores all data necessary for a simulation. Most attributes of the
	class are instances of the class ConfigItem whose values are set to an
	empty string. Another attribute is a list of all other attributes
	which allows to access them easily and automatically. The main aim of
	this class is to enables the user to save the configuration in a file
	or to load a configuration file.
	"""
	
	def __init__(self):
		"""
		Constructor.  Sets all values to a null string.
		"""
		self.path = ConfigItem("Polyphemus_path", "[paths]", "")
		self.meteo = ConfigItem("Meteorological_file", "[paths]", "")
		self.data = ConfigItem("Raw_data", "[paths]", "" )
		self.date1 = ConfigItem("From", "[domain]", "")
		self.date2 = ConfigItem("To", "[domain]", "")
		self.x_min = ConfigItem("x_min", "[domain]", "")
		self.Delta_x = ConfigItem("Delta_x", "[domain]", "")
		self.Nx = ConfigItem("Nx", "[domain]", "")
		self.y_min = ConfigItem("y_min", "[domain]", "")
		self.Delta_y = ConfigItem("Delta_y", "[domain]", "")
		self.Ny = ConfigItem("Ny", "[domain]", "")
		self.Nz = ConfigItem("Nz", "[domain]", "")
		self.levels = ConfigItem("Levels", "[domain]", "")
		self.species = ConfigItem("Species", "[species]", "")
		self.source = ConfigItem("Source_file", "[species]", "")
		self.initial = ConfigItem("Initial_quantity", "[species]","")
		self.parameter = ConfigItem("Kz_parameterization", "[Kz]", "")
		
		self.list = [self.path, self.meteo, self.data, self.date1,
			     self.date2, self.x_min, self.Delta_x, self.Nx,
			     self.y_min, self.Delta_y, self.Ny, self.Nz,
			     self.levels, self.species, self.source,
			     self.initial, self.parameter]

	def load(self, filename):
		"""
		Loads a configuration file. The file is read and the value of
		each ConfigItem is modified in accordance to the values given
		in the file.

		@type filename: String.
		@param filename: The name of the file to load.
		"""
		import talos
		config_file = talos.ConfigStream(filename)
		for item in self.list:
			item.value = config_file.GetString(item.name, item.section)



class PolyphemusDisplay(FigureCanvas):
	"""
	Displays concentrations on a Basemap.
	"""
	
	def __init__(self, parent, width, height, dpi):
		"""
		Constructor.

		@type parent: QWidget.
		@param parent: The parent widget of PolyphemusDisplay.

		@type width: integer.
		@param width: The width of the figure in inches. 
		
		@type height: integer.
		@param height: The height of the figure in inches. 

		@type dpi: integer.
		@param dpi: The number of dots per inches in the
		figure. 
		"""
		self.width = width
		self.height = height
		self.dpi = dpi
		self.fig = figure(figsize=(self.width, self.height),
				  dpi = self.dpi, facecolor = 'w',
				  edgecolor = 'w')
		self.axes = self.fig.add_axes([0, 0, 1, 1])
		FigureCanvas.__init__(self, self.fig)
		self.reparent(parent, QPoint(0, 0))
		xticks([])
		yticks([])
		self.configuration = ""  # configuration file used to display.
		self.step = 0            # time step displayed.
		self.level = 0           # level displayed
		self.map = None
		self.data = None
		FigureCanvas.setSizePolicy(self,
					   QSizePolicy.Expanding,
					   QSizePolicy.Expanding)
		FigureCanvas.updateGeometry(self)
		self.col = None

	def dispcf(self):
		"""
		Displays a binary file on a map using talos functions.
		The binary file and domain description are given in the
		attribute 'config' of the class.
		"""
		import talos
		
		configuration = talos.Config(self.config)
		
		if self.map is None: 
			self.map = display.getm(self.config, cbar = False) 
		if self.data is None:
			self.data = display.getd(self.config)
			
		self.fig.clf()
		self.axes.cla()
		if self.col != None:
			del self.col
		
		xrange, yrange = self.map.makegrid(self.data.shape[3],
						   self.data.shape[2])		
		self.map.contourf(xrange, yrange,
				  self.data[self.step, self.level])
		self.map.drawcountries()
		self.map.drawcoastlines()

		self.col = colorbar(orientation = 'horizontal')
		self.setPaletteBackgroundColor(QColor("white"))
		self.draw()
		

	def clear(self):
		"""Clears the figure."""
		self.fig.clf()
		
	

class LineEdit(QLineEdit):
	"""
	Stores a value.
	The friend of a LineEdit is a ConfigItem whose value is displayed in
	the line. There is a two way relationship between a LineEdit and its
	friend: when the value of one (the value of a LineEdit being the value
	displayed) is modified by the user, the value of the other is modified
	too so that they always match. Functions 'print_friend' and 'update'
	represent this relationship.
	"""
	
	def __init__(self, content, parent, friend):
		"""
		Constructor.

		@type content: string.
		@param content: The text entered in the line when it is
		created. 

		@type parent: QWidget.
		@param top: The parent widget.

		@type friend: ConfigItem.
		@param friend: The ConfigItem whose value managed in the line
		edit. 
		"""
		QLineEdit.__init__(self, content, parent, "")
		self.setAlignment(Qt.AlignLeft)
		self.friend = friend
		self.connect(self, SIGNAL("textChanged(const QString &)"),
			     self.update)
		
	def print_friend(self):
		"""
		Changes the value displayed on the line when the value of the
		friend is modified. 
		"""
		self.setText(str(self.friend.value))

	def update(self, s):
		"""
		Changes the value of the friend when the value on the line
		edit is modified.
		"""
		if s != "":
			self.friend.value = str(s)
			


class FileSelector(QWidget):
	"""
	Allows the user to select a file.
	Contains a text, a LineEdit whose content is the file selected and a
	button which opens a QFileDialog. There is thus two ways to select a
	file: either by entering its path and name in the LineEdit or with the
	file dialog.
	"""

	def __init__(self, content, parent, friend, filter, caption):
		"""
		Constructor.

		@type content: string.
		@param content: The text entered in the line when it is
		created. 

		@type parent: QWidget.
		@param top: The parent widget.

		@type friend: ConfigItem.
		@param friend: The ConfigItem whose value managed in the line
		edit. 

		@type filter: String.
		@param filter: The filter used when selecting the file (for
		example 'Data files (*.dat)').

		@type caption: String.
		@param caption: The caption displayed when the dialog is
		opened. 
		"""
		QWidget.__init__(self, parent)
		self.friend = friend
		self.filter = filter
		self.caption = caption
		self.grid = QGridLayout(self, 1, 3)
		self.label = QLabel(content, self)
		self.grid.addWidget(self.label, 0, 0)
		self.line = LineEdit("", self, self.friend)
		self.grid.addWidget(self.line, 0, 1)
		self.button = QPushButton(self)
		self.button.setPixmap(QPixmap("images/dossier-ouvert.png"))
		self.grid.addWidget(self.button, 0, 2)
		self.connect(self.button, SIGNAL("clicked()"), self.select)

	def select(self):
		"""
		Opens a file dialog and modifies the content off the line
		edit to match the file selected.
		"""
	        f = self.line.text()
		if f != "":
			s = QFileDialog.getOpenFileName(f, self.filter, self,
							"", self.caption)
		else:
			s = QFileDialog.getOpenFileName("./", self.filter,
							self, "",
							self.caption)
		try:
			self.line.setText(str(s))
		except:
			pass
		
	def print_friend(self):
		"""
		Changes the value displayed on the line edit when the value of the
		friend is modified. 
		"""
		self.line.print_friend()
		
	def clear(self):
		"""
		Clears the content of the line edit.
		"""
		self.line.clear()


class DirSelector(QWidget):
	"""
	Similar to class FileSelector except that the file dialog only
	allows to select a directory.
	"""
	
	def __init__(self, content, parent, friend, caption):
		"""
		Constructor.

		@type content: string.
		@param content: The text entered in the line when it is
		created. 

		@type parent: QWidget.
		@param parent: The parent widget.

		@type friend: ConfigItem.
		@param friend: The ConfigItem whose value is managed in the line
		edit. 

		@type caption: String.
		@param caption: The caption displayed when the dialog is
		opened. 
		"""
		QWidget.__init__(self, parent)
		self.friend = friend
		self.caption = caption
		self.grid = QGridLayout(self, 1, 3)
		self.label = QLabel(content, self)
		self.grid.addWidget(self.label, 0, 0)
		self.line = LineEdit("", self, self.friend)
		self.grid.addWidget(self.line, 0, 1)
		self.button = QPushButton(self)
		self.button.setPixmap(QPixmap("images/dossier-ouvert.png"))
		self.grid.addWidget(self.button, 0, 2)
		self.connect(self.button, SIGNAL("clicked()"), self.select)

	def select(self):
		"""
		Opens a file dialog and modifies the content of the line
		edit to match the directory selected.
		"""
		f = self.line.text()
		if f != "":
			s = QFileDialog.getExistingDirectory(f, self, "",
							     self.caption,
							     True, False)
		else:
			s = QFileDialog.getExistingDirectory("./", self, "",
							     self.caption,
							     True, False)
		try:
			self.line.setText(str(s))
		except:
			pass

	def print_friend(self):
		"""
		Changes the value displayed on the line edit when the value of the
		friend is modified. 
		"""
		self.line.print_friend()	

	def clear(self):
		"""
		Clears the content of the line edit.
		"""
		self.line.clear()


class SectionClass(QWidget):
	"""
	Class used to define some parameters of the simulation.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type config: PolyphemusConfig.
		@param config: The configuration used to define the various
		line edits' friends.
		"""
		QWidget.__init__(self, parent)
		self.config = config
		self.lineList = []

	def print_friend(self):
		"""
		Changes the value displayed on the lines when the value of
		their friends is modified.  
		"""
		for l in self.lineList:
			l.print_friend()

	def clear(self):
		"""
		Clears the content of every line edit of the section.
		"""
		for line in self.lineList:
			line.clear()	


		
class DialogFile(QDialog):
	"""
	Class used to open a temporary dialog which allows to reload the
	source file.
	"""
	
	def __init__(self, top, parent, friend, filename, content):
		"""
		Constructor.

		@type top: QWidget.
		@param top: The top-level widget. It is used to modify the
		source file.

		@type parent: QWidget.
		@param parent: The parent widget.

		@type friend: LineEdit.
		@param friend: The LineEdit whose value can be modified.

		@type filename: String.
		@param: the file to load.

		@type content: String.
		@param content: The text to display in the dialog.
		"""
		QDialog.__init__(self, parent)
		self.setModal(True)
		self.top = top
		self.friend = friend
		self.file = filename
		self.grid = QGridLayout(self, 2, 4)
		self.label = QLabel(content, self)
		self.grid.addMultiCellWidget(self.label, 0, 0, 0, 3)
		self.pushYes = QPushButton ("Yes", self)
		self.grid.addWidget(self.pushYes, 1, 0)
		self.pushNo = QPushButton ("No", self)
		self.grid.addWidget(self.pushNo, 1, 3)
		self.connect(self.pushYes, SIGNAL("clicked()"), self.load)
		self.connect(self.pushNo, SIGNAL("clicked()"), self.close)

	def load(self):
		"""
		Allows the user to select the file (in case it was saved with
		another name) and loads it.
		"""
		s = QFileDialog.getOpenFileName(self.file,
						"Data file(*.dat);;All files (*)",
						self, "",
						"Select the source file to load.")
		try:
			self.friend.setText(str(s))	
		except:
			pass
		self.close()	


class HelpBox(QDialog):
	"""
	Class used to open a temporary dialog with information.
	"""
	
	def __init__(self, parent, textfile):
		"""
		Constructor.
		
		@type parent: QWidget.
		@param parent: The parent widget.

		@type textfile: String.
		@param textfile: The file to display in the help box.
		"""
		QDialog.__init__(self, parent)
		self.setModal(False)
		self.setPaletteBackgroundColor(QColor("white"))
		self.grid = QGridLayout(self, 3, 3)
		self.grid.setRowSpacing(1, 20)
		self.text = QTextBrowser(self)
		self.text.mimeSourceFactory().addFilePath(".")
		self.text.mimeSourceFactory().addFilePath("help/")
		self.text.setSource(textfile)
		self.text.show()
		self.grid.addMultiCellWidget(self.text, 0, 0, 0, 2)
		self.push = QPushButton ("Close", self)
		self.grid.addWidget(self.push, 2, 1)
		self.connect(self.push, SIGNAL("clicked()"), self.close)
		self.show()
		


# Classes to create all elements of the window. This classes exist so that the
# modification of an element can be done outside the creation of the
# MainWindow.
		

class SelectPath(DirSelector):
	"""
	DirSelector to select the path to Polyphemus.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
		The values of content and captions are given.
		"""
		DirSelector.__init__(self, "Path to Polyphemus", parent,
		config.path, "Select the path to Polyphemus directory")

	def resize(self, size):
		sp = size.width() / 6
		self.grid.setColSpacing(0, 2 * sp)
		self.grid.setColSpacing(1, 3 * sp)
		self.grid.setColSpacing(2, sp)

		
class SectionDomain(SectionClass):
	"""
	SectionClass to define the domain.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type config: PolyphemusConfig.
		@param config: The configuration used to define the various
		line edits' friends.
		"""
		SectionClass.__init__(self, parent, config)
		self.grid = QGridLayout(self, 3, 6)

		width = self.width() / 6
		for i in range(6):
			self.grid.setColSpacing(i, width)

		self.labelxmin = QLabel("x_min = ", self)
		self.grid.addWidget(self.labelxmin, 0, 0)
		self.lineEditxmin = LineEdit("", self, self.config.x_min)
		self.grid.addWidget(self.lineEditxmin, 0, 1)
		self.labelDeltax = QLabel("Delta x = ", self)
		self.grid.addWidget(self.labelDeltax, 0, 2)
		self.lineEditDeltax = LineEdit("", self, self.config.Delta_x)
		self.grid.addWidget(self.lineEditDeltax, 0, 3)
		self.labelNx = QLabel("Nx = ", self)
		self.grid.addWidget(self.labelNx, 0, 4)
		self.lineEditNx = LineEdit("", self, self.config.Nx)
		self.grid.addWidget(self.lineEditNx, 0, 5)
		
		self.labelymin = QLabel("y_min = ", self)
		self.grid.addWidget(self.labelymin, 1, 0)
		self.lineEditymin = LineEdit("", self, self.config.y_min)
		self.grid.addWidget(self.lineEditymin, 1, 1)
		self.labelDeltay = QLabel("Delta y = ", self)
		self.grid.addWidget(self.labelDeltay, 1, 2)
		self.lineEditDeltay = LineEdit("", self, self.config.Delta_y)
		self.grid.addWidget(self.lineEditDeltay, 1, 3)
		self.labelNy = QLabel("Ny = ", self)
		self.grid.addWidget(self.labelNy, 1, 4)
		self.lineEditNy = LineEdit("", self, self.config.Ny)
		self.grid.addWidget(self.lineEditNy, 1, 5)
		
		self.labelNz = QLabel("Nz = ", self)
		self.grid.addWidget(self.labelNz, 2, 0)
		self.lineEditNz = LineEdit("", self, self.config.Nz)
		self.grid.addWidget(self.lineEditNz, 2, 1)

		self.selectLevels = FileSelector("Levels :", self,
		self.config.levels, "Data file (*.dat);; All files (*)",
		"Select the levels file")
		self.grid.addMultiCellWidget(self.selectLevels, 2, 2, 3, 5)

		self.lineList = [self.lineEditxmin, self.lineEditDeltax,
				  self.lineEditNx, self.lineEditymin,
				  self.lineEditDeltay, self.lineEditNy,
				  self.lineEditNz, self.selectLevels.line]


class SectionDate(SectionClass):
	"""
	SectionClass to define the dates.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type config: PolyphemusConfig.
		@param config: The configuration used to define the friend of
		the line edit (config.date).
		"""
		SectionClass.__init__(self, parent, config)
		self.grid = QGridLayout(self, 1, 5)
		self.label_from = QLabel("From: ", self)
		self.grid.addWidget(self.label_from, 0, 0)
		self.line_from = LineEdit("", self, self.config.date1)
		self.grid.addWidget(self.line_from, 0, 1)
		self.label_to = QLabel("To: ", self)
		self.grid.addWidget(self.label_to, 0, 3)
		self.line_to = LineEdit("", self, self.config.date2)
		self.grid.addWidget(self.line_to, 0, 4)
		self.lineList = [self.line_from, self.line_to]


class SelectMeteo(FileSelector):
	"""
	DirSelector to select the path to raw data.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type config: PolyphemusConfig.
		@param config: The configuration used to define the friend of
		the line edit (config.data).
		"""
		FileSelector.__init__(self, "MM5 file: ", parent,
		config.meteo, "All files (*)", "Select the path to the MM5 file.")
		

class SectionSpecies(SectionClass):
	"""
	SectionClass to define the species used, its sources and its initial
	condition. Note that only one species can be considered.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type config: PolyphemusConfig.
		@param config: The configuration used to define the friends of
		the various line edits.
		"""
		SectionClass.__init__(self, parent, config)
		self.grid = QGridLayout(self, 2, 5)

		width = self.width() / 5
 		for i in range(5):
 			self.grid.setColSpacing(i, width)

		
		self.labelSpecies = QLabel("Species: ", self)
		self.grid.addWidget(self.labelSpecies, 0, 0)
		self.lineEditSpecies = LineEdit("", self, self.config.species)
		self.grid.addWidget(self.lineEditSpecies, 0, 1)
		self.selectSource = FileSelector("Source:", self,
		self.config.source, "Data file (*.dat);; All files (*)",
		"Select the source file")
		self.grid.addMultiCellWidget(self.selectSource, 0, 0, 3, 4)
		
		self.labelInit = QLabel("Initial quantity: ", self)
		self.grid.addWidget(self.labelInit, 1, 0)
		self.lineEditInit = LineEdit("", self, self.config.initial)
		self.grid.addWidget(self.lineEditInit, 1, 1)
		self.lineList = [self.lineEditSpecies, self.selectSource.line,
				 self.lineEditInit]

class SectionKz(SectionClass):
	"""
	SectionClass to define the Kz parameterization.
	"""
	
	def __init__(self, parent, config):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type config: PolyphemusConfig.
		@param config: The configuration used to define the friend of
		the line edit (config.param).
		"""
		SectionClass.__init__(self, parent, config)
		self.grid = QGridLayout(self, 1, 3)
		self.friend = config.parameter
		self.label = QLabel("Kz parameterization: ", self)
		self.grid.addWidget(self.label, 0, 0)
		self.comboBoxKz = QComboBox(False, self)
		self.comboBoxKz.insertItem("Louis")
		self.comboBoxKz.insertItem("Troen and Mahrt")
		self.grid.addMultiCellWidget(self.comboBoxKz, 0, 0, 1, 2)
		self.lineList=[]
		self.connect(self.comboBoxKz,
			     SIGNAL("textChanged(const QString &)"),
			     self.update)

	def print_friend(self):
		"""
		Changes the value displayed on the combo box when the value of
		the friend is modified.
		"""
		if self.friend.value == "TM":
			self.comboBoxKz.setCurrentText("Troen and Mahrt")
		else:
			self.comboBoxKz.setCurrentText("Louis")

	def update(self, s):
		"""
		Changes the value of the friend when the value on the combo
		box is modified.
		"""
		if s == "Troen and Mahrt":
			s = "TM"
		self.friend.value = s

	def clear(self):
		"""
		Reset the combo box to its default value ('Louis')
		"""
		self.comboBoxKz.setCurrentText("Louis")
		

class ImageViewer(QWidget):
	"""
	Holds a PolyphemusDisplay and lines and buttons to control the figure
	displayed.
	"""
	
	def __init__(self, parent, top, width, height, dpi):
		"""
		Constructor.
				
		@type parent: QWidget.
		@param parent: The parent widget.

		@type height: integer.
		@param height: The height of the figure in inches. Default
		value is 5.

		@type width: integer.
		@param width: The width of the figure in inches. Default
		value is 4.
		
		@type dpi: integer.
		@param dpi: The number of dots per inches in the
		figure. Default value is 100. 
		"""
		QWidget.__init__(self, parent)
		# Creation of the layout and of its content.
		self.grid = QGridLayout(self, 5, 4)
		width = width * dpi / 4
		for i in range(4):
			self.grid.setColSpacing(i, width)
		self.top = top
		self.labelDate = QLabel("Date shown: ", self)
		self.grid.addWidget(self.labelDate, 0, 0)
		self.lineDate = QLineEdit("", self)
		self.lineDate.setAlignment(Qt.AlignLeft)
		self.lineDate.setReadOnly(True)
		self.grid.addMultiCellWidget(self.lineDate, 0, 0, 1, 2)
		self.polyphemusDisplay = PolyphemusDisplay(self, width, height, dpi)
		self.grid.addMultiCellWidget(self.polyphemusDisplay, 1, 1, 0, 3)
		self.labelStep = QLabel("Visualization for time step: ", self)
		self.grid.addMultiCellWidget(self.labelStep, 2, 2, 1, 2)
		self.lineEditStep = QLineEdit("0", self)
		self.grid.addWidget(self.lineEditStep, 2, 3)
		self.labelLevel = QLabel("Visualization for level: ", self)
		self.grid.addMultiCellWidget(self.labelLevel, 3, 3, 1, 2)
		self.lineEditLevel = QLineEdit("0", self)
		self.grid.addWidget(self.lineEditLevel, 3, 3)
		self.pushBackward = QPushButton(" << ", self)
		self.pushBackward.setAccel(Qt.CTRL + Qt.Key_PageUp)
		self.grid.addMultiCellWidget(self.pushBackward, 4, 4, 0, 1)
		self.pushForward = QPushButton(" >> ", self)
		self.pushForward.setAccel(Qt.CTRL + Qt.Key_PageDown)	
		self.grid.addMultiCellWidget(self.pushForward, 4, 4, 2, 3)

		# Connection of the signal and slots.
		self.connect(self.pushBackward, SIGNAL("clicked()"),
			     self.backward)
		self.connect(self.pushForward, SIGNAL("clicked()"),
			     self.forward)
		self.connect(self.lineEditLevel,
			     SIGNAL("textChanged(const QString &)"),
			     self.disp)
		self.connect(self.lineEditStep,
			     SIGNAL("textChanged(const QString &)"),
			     self.disp)

	def print_friend(self):
		"""
		Changes the value displayed on the lines when the value of
		their friends is modified. 
		"""
		self.lineEditStep.setText(str(self.polyphemusDisplay.step))
		self.lineEditLevel.setText(str(self.polyphemusDisplay.level))
		self.date = str(self.top.date.line_from.text())
		date = AddHours(self.date, self.polyphemusDisplay.step)
		self.lineDate.setText(date)

	def disp(self, s):
		"""
		Display the figure when the value on the lines are modified.

		@type s: String.
		@param s: The string entered in one of the line. The function
		only checks that it is not a null string.
		"""
		if s != "":
			st = self.lineEditStep.text().toInt()[0]
			self.polyphemusDisplay.step = st
			lev = self.lineEditLevel.text().toInt()[0]
			self.polyphemusDisplay.level = lev
			self.polyphemusDisplay.dispcf()
		self.lineDate.setText(AddHours(self.date,
					       self.polyphemusDisplay.step))
		
	def forward(self):
		"""
		Increases the value of PolyphemusDisplay.step by one if
		possible and display the updated figure. If it is not
		possible, it does nothing.
		"""
		M = self.polyphemusDisplay.data.shape[0] - 1
		if self.polyphemusDisplay.step != M:
			self.polyphemusDisplay.step += 1
			self.polyphemusDisplay.dispcf()
			self.print_friend()

	def backward(self):
		"""
		Decreases the value of PolyphemusDisplay.step by one if
		possible and display the updated figure. If it is not
		possible, it does nothing.
		"""
		if self.polyphemusDisplay.step != 0:
			self.polyphemusDisplay.step -= 1
			self.polyphemusDisplay.dispcf()
			self.print_friend()
			
	def clear(self):
		"""
		Clears the ImageViewer.
		"""
		self.lineEditStep.setText("0")
		self.lineEditLevel.setText("0")
		self.polyphemusDisplay.clear()
			


class GroupButton(QWidget):
	"""
	Creates four buttons to launch the different parts of the simulation,
	clear the dialog and quit the dialog.
	"""
	
	def __init__(self, top, parent):
		"""
		Constructor.

		@type top: QWidget.
		@param top: The top-level widget. It is used for the
		connection of signals and slots.

		@type parent: QWidget.
		@param parent: The parent widget.
		"""
		QWidget.__init__(self, parent)
		self.top = top
		self.grid = QGridLayout(self, 1, 4)
		width = self.width() / 4
		for i in range(4):
			self.grid.setColSpacing(i, width)
		self.pushPreprocess = QPushButton("&Preprocessing", self)
		self.pushPreprocess.setAccel(Qt.CTRL + Qt.Key_P)
		self.grid.addWidget(self.pushPreprocess, 0, 0)
		self.pushSimulation = QPushButton("Simula&tion", self)
		self.pushSimulation.setAccel(Qt.CTRL + Qt.Key_T)
		self.grid.addWidget(self.pushSimulation, 0, 1)
		self.pushClear = QPushButton("Cl&ear", self)
		self.grid.addWidget(self.pushClear, 0, 2)
		self.pushQuit = QPushButton("&Quit", self)
		self.grid.addWidget(self.pushQuit, 0, 3)

		self.connect(self.pushPreprocess, SIGNAL("clicked()"),
			     self.top.preprocess)
		self.connect(self.pushSimulation, SIGNAL("clicked()"),
			     self.top.simulation)
		self.connect(self.pushClear, SIGNAL("clicked()"),
 			     self.top.clear)
 		self.connect(self.pushQuit, SIGNAL("clicked()"),
 			     self.top.close)
		

class MainWindow(QMainWindow):
	"""
	Main window of the application.
	"""
	
	def __init__(self, parent = None, name = None):
		"""
		Constructor.

		@type parent: QWidget.
		@param parent: The parent widget (optional).

		@type name: String.
		@param name: The name of the window (optional).
		"""
		# Creation of the main window and its menu bar.
		QMainWindow.__init__(self, parent, name)
		self.main = QWidget(self)
		self.setCentralWidget(self.main)
		self.setCaption("polyphemus_display.py")
		self.setIcon(QPixmap("images/logo.png"))

		self.file = QPopupMenu(self, "file")
		self.menuBar().insertItem('&File', self.file)
		self.file.insertItem("&Save configuration file",
		self.save_config, Qt.CTRL + Qt.Key_S)
		self.file.insertItem("L&oad configuration file",
		self.load_config, Qt.CTRL + Qt.Key_O)
		self.file.insertItem("Cl&ear", self.clear,
				     Qt.CTRL + Qt.Key_E)
		self.file.insertItem("&Quit", self.close,
				     Qt.CTRL + Qt.Key_Q)

		self.edit = QPopupMenu(self, "edit")
		self.menuBar().insertItem('Edit', self.edit)
		self.edit.insertItem('Edit sou&rce file', self.edit_source,
				       Qt.CTRL + Qt.Key_R)
		self.edit.insertItem('Edit &levels file',
		self.edit_levels, Qt.CTRL + Qt.Key_L)

		self.help =  QPopupMenu(self, "help")
		self.menuBar().insertItem('Help', self.help)
		self.help.insertItem("&About", self.about, Qt.CTRL + Qt.Key_A)
		self.help.insertItem("&Help", self.help_box, Qt.CTRL + Qt.Key_H)
	
		self.statusBar().message("Launch the help file with Ctrl+H.")

		self.config = PolyphemusConfig()
		
		self.add_elements()
		self.resize_grid()
		
		sectionList = [self.domain, self.date, self.species]
		for section in sectionList:
			for line in section.lineList:
				self.connect(self,
					     PYSIGNAL("do_print_friend()"),
					     line.print_friend)

	def add_elements(self):
		"""
		Adds the various elements of the window to the grid.
		"""
		self.grid = QGridLayout(self.main, 13, 7)
		
		self.selectPath = SelectPath(self.main, self.config)
		self.grid.addMultiCellWidget(self.selectPath, 0, 0, 2, 6)

		self.image = ImageViewer(self.main, self, 5, 5, 100)
		self.grid.addMultiCellWidget(self.image, 2, 12, 1, 2)

		self.domain = SectionDomain(self.main, self.config)
		self.grid.addMultiCellWidget(self.domain, 2, 2, 4, 5)

		self.meteo = SelectMeteo(self.main, self.config)
		self.grid.addMultiCellWidget(self.meteo, 4, 4, 4, 5)

		self.date = SectionDate(self.main, self.config)
		self.grid.addMultiCellWidget(self.date, 6, 6, 4, 5)

		self.species = SectionSpecies(self.main, self.config)
		self.grid.addMultiCellWidget(self.species, 8, 8, 4, 5)

		self.kz = SectionKz(self.main, self.config)
		self.grid.addWidget(self.kz, 10, 4)
     
		self.button = GroupButton(self, self.main)
		self.grid.addMultiCellWidget(self.button, 12, 12, 4, 5)

	def resize_grid(self):
		"""
		Resizes the widget and the columns and rows of its layout.
		"""
		self.resize(QSize(740, 640))

		# Defines the minimum width of the columns.
		self.grid.setColSpacing(0, 10)
		self.grid.setColSpacing(1, 150)
		self.grid.setColSpacing(2, 150)
		self.grid.setColSpacing(3, 20)
		self.grid.setColSpacing(4, 200)
		self.grid.setColSpacing(5, 200)
		self.grid.setColSpacing(6, 10)

		# Defines the minimum height of the rows.
		self.grid.setRowSpacing(0, 40)
		self.grid.setRowSpacing(1, 40)
		self.grid.setRowSpacing(2, 120)
		self.grid.setRowSpacing(3, 40)
		self.grid.setRowSpacing(4, 40)
		self.grid.setRowSpacing(5, 40)
		self.grid.setRowSpacing(6, 40)
		self.grid.setRowSpacing(7, 40)
		self.grid.setRowSpacing(8, 80)
		self.grid.setRowSpacing(9, 40)
		self.grid.setRowSpacing(10, 40)
		self.grid.setRowSpacing(11, 40)
		self.grid.setRowSpacing(12, 40)

		

	def save_config(self):
		"""
		Saves all parameters of the simulation in a configuration
		file. If the file is not found, nothing is done.
		"""
		import talos
		try:
			f = str(QFileDialog(self.file).getSaveFileName("./",
			"", self, "f_dial", "Select the file in which to save."))
		
			if self.kz.comboBoxKz.currentText() == "Troen and Mahrt":
				self.config.parameter.value = "TM"
			else:
				self.config.parameter.value = "Louis"	
			
			config = []
			for item in self.config.list:
				config.append([item.name, item.section,
					       item.value])
			talos.write_configuration_file(config, f)
			
		except:
			pass

	def load_config(self):
		"""
		Loads all parameters of the simulation from a configuration
		file. If the file is not found, nothing is done.
		"""
		try:
			f = str(QFileDialog(self.file).getOpenFileName(
				"./configuration.cfg", "", self, "f_dial",
				"Select the configuration file to open."))
			self.config.load(f)
			self.kz.print_friend()
			self.selectPath.print_friend()
			self.meteo.print_friend()
			self.config.data.value = os.path.dirname(self.config.meteo.value)
			self.emit(PYSIGNAL("do_print_friend()"), ())
		except:
			pass

	def edit_source(self):
		"""
		Opens the source file in emacs, allows the user to modify it
		then reloads it if necessary. 
		"""
		import commands
		f = str(QFileDialog(self.edit).getOpenFileName("./",
		"Data file (*.dat);; All files (*)", self, "",
		"Select the source file to edit."))
		
		if f != "":
			commands.getstatusoutput("emacs " + f)
			friend = self.species.selectSource.line
			dial = DialogFile(self, self.main, friend, f,
			"Source file has been modified. Do you wish to reload it?")
			dial.show()

	def edit_levels(self):
		"""
		Opens the levels file in emacs, allows the user to modify it
		then reloads it if necessary. 
		"""
		import commands
		f = str(QFileDialog(self.edit).getOpenFileName("./",
		"Data file (*.dat);; All files (*)", self, "f_dial",
		"Select the levels file to edit."))
		
		if f !="":
			commands.getstatusoutput("emacs " + f)
			friend = self.domain.selectLevels.line
			dial = DialogFile(self, self.main, friend, f,
			"Levels file has been modified. Do you wish to reload it?")
			dial.show()


	def help_box(self):
		"""
		Opens a dialog to display help information.
		"""
		help = HelpBox(self.main, "help/help.txt")
		help.setCaption("polyphemus-disp.py Help File.")
		help.resize(QSize(900, 800))
		

	def about(self):
		"""
		Opens a dialog to display copyright information.
		"""
		about = HelpBox(self.main, "help/about.txt")
		about.setCaption("About polyphemus-disp.py.")
		about.resize(QSize(650, 340))


	def preprocess(self):
		"""
		Performs preprocessing.
		"""
		import commands, shutil
		
		self.statusBar().message("Preprocessing")
		
		poly_path = self.config.path.value
	    
		Date1 = self.config.date1.value
 		Date2 = self.config.date2.value

		# We add 5 hours to the second date to be sure that
		# preprocessing is sufficient.
		Date2 = AddHours(Date2, 5)
		
		# Creation of the simulation.
		simulation = Polyphemus(self.config.species.value)
            
		# Working directories.
		ref_config_dir = "configuration/"
		
		working_dir = "work/"
		
		config_dir = working_dir + "configuration/"
		data_dir = working_dir + "preprocessing/"
		
		# Delete and re-create results directory.
		if os.path.isdir(data_dir):
			shutil.rmtree(data_dir)
		os.mkdir(data_dir)
		os.mkdir(data_dir + "meteo/")
		os.mkdir(data_dir + "ground/")

		meteo_file = self.config.meteo.value
		self.config.data.value = os.path.dirname(meteo_file)[0:-4] + "/"

		# Dictionary used to replace values in all configuration files.
		preprocess_dict = {"%Date1%": Date1, "%Date2%": Date2, 
				   "%x_min%": self.config.x_min.value,
				   "%Delta_x%": self.config.Delta_x.value,
				   "%Nx%": self.config.Nx.value,
				   "%y_min%": self.config.y_min.value,
				   "%Delta_y%": self.config.Delta_y.value,  
				   "%Ny%": self.config.Ny.value,
				   "%Nz%": self.config.Nz.value,
				   "%levels%" : self.config.levels.value,
				   "%species%": self.config.species.value,
				   "%quantity_ic%": self.config.initial.value, 
				   "%source%": self.config.source.value,
				   "%param%": self.config.parameter.value,
				   "%meteo_file%": meteo_file,
				   "%raw_data%": self.config.data.value[:-1],
				   "%config_dir%": config_dir[:-1], 
				   "%data_dir%": data_dir[:-1],
				   "%polyphemus_path%": poly_path[:-1]}

		config = Configuration([ref_config_dir + "species.dat",
					ref_config_dir + "levels.dat",
					ref_config_dir + "land_data_usgs.dat",
					ref_config_dir + "roughness-usgs.dat"])
		config.SetConfiguration(preprocess_dict, "path", config_dir)
		config.ProceedReplacement()	
        
		# luc-usgs.
		program = Program(poly_path + 
				  "preprocessing/ground/luc-usgs",
				  format = " %a ")
		program.SetConfiguration([ref_config_dir + "general.cfg",
					  ref_config_dir + "luc-usgs.cfg"])
		program.SetConfiguration(preprocess_dict, "path", config_dir)
		simulation.AddProgram(program)

            
		# roughness.
		program = Program(poly_path + 
				  "preprocessing/ground/roughness",
				  format = " %a ")
		program.SetConfiguration([ref_config_dir +"general.cfg",
					  ref_config_dir + "roughness.cfg"])
		program.SetConfiguration(preprocess_dict, "path", config_dir)
		simulation.AddProgram(program)
	    
		# MM5-meteo.
		program = Program(poly_path +
				  "preprocessing/meteo/MM5-meteo",
				  format = " %a " + Date1 + " " + Date2)
		program.SetConfiguration([ref_config_dir + "general.cfg",
					  ref_config_dir + "MM5-meteo.cfg"])
		program.SetConfiguration(preprocess_dict, "path", config_dir)
		simulation.AddProgram(program)
        
		# Kz_TM if necessary.
		if self.config.parameter.value == "TM":
			program = Program(poly_path +
					  "preprocessing/meteo/Kz_TM",
					  format = " %a " + Date1 + " " + Date2)
			program.SetConfiguration([ref_config_dir +
						  "general.cfg",
						  ref_config_dir +
						  "MM5-meteo.cfg"]) 
			program.SetConfiguration(preprocess_dict,
						 "path", config_dir)
			simulation.AddProgram(program)

		# Launch of the simulation.
		simulation.Run()

			

	def simulation(self):
		"""
		Performs a simulation using results of self.preprocessing.
		"""
		import commands, shutil

		self.statusBar().message("Diffusion")

		poly_path = self.config.path.value
	
		Date1 = self.config.date1.value
 		Date2 = self.config.date2.value

		Nt = compute_Nt(Date1, Date2, 600)
		Nt_save = Nt / 6
            
		# Creation of the simulation.
		simulation = Polyphemus(self.config.species.value)
            
		# Working directories.
		ref_config_dir = "configuration/"

		working_dir = "work/"
	    
		config_dir = working_dir + "configuration/"
		data_dir = working_dir + "preprocessing/"
		result_dir = working_dir + "results/"
		
		# Delete and re-create results directory.
		if os.path.isdir(result_dir):
			shutil.rmtree(result_dir)
		os.mkdir(result_dir)
		
		# Levels for which concentrations are saved: all levels used.
		Nzr = int(self.config.Nz.value)
		zrange = range(Nzr)
		levels_save = ""
		for k in zrange:
			levels_save += str(k) + " "

		simulation_dict = {"%Date1%": Date1, "%Date2%": Date2,
				   "%x_min%": self.config.x_min.value,
				   "%Delta_x%": self.config.Delta_x.value,
				   "%Nx%": self.config.Nx.value,
				   "%y_min%": self.config.y_min.value,
				   "%Delta_y%": self.config.Delta_y.value,  
				   "%Ny%": self.config.Ny.value,
				   "%Nz%": self.config.Nz.value,
				   "%Nt_simu%": Nt,
				   "%Nt_save%": Nt_save,
				   "%levels%" : self.config.levels.value,
				   "%levels_save%": levels_save,
				   "%species%": self.config.species.value,
				   "%quantity_ic%": self.config.initial.value,
				   "%param%": self.config.parameter.value,
				   "%source%": self.config.source.value,
				   "%config_dir%": config_dir[:-1], 
				   "%result%": result_dir[:-1],
				   "%data_dir%": data_dir[:-1],
				   "%polyphemus_path%": poly_path[:-1]}

	
		# polair3D.
		program = Program(poly_path + "driver/polair3d",
				  format = " %a")
		program.SetConfiguration(ref_config_dir + "polair3d.cfg")
		program.SetConfiguration(simulation_dict, "path", config_dir)
		
		# Additional configuration files.
		config = Configuration([ref_config_dir + "species.dat",
					ref_config_dir + "polair3d-data.cfg",
					ref_config_dir + "polair3d-saver.cfg",
					ref_config_dir + "disp.cfg"])
		config.SetConfiguration(simulation_dict, "path", config_dir)
		config.ProceedReplacement()
		
		simulation.AddProgram(program)

		# Launch of the simulation.
		simulation.Run()

		# Display of results.
		self.statusBar().message("Displaying results")
		config_disp = Configuration(ref_config_dir + "disp.cfg")
		config_disp.SetConfiguration(simulation_dict, "path", result_dir)
		config_disp.ProceedReplacement()

		config_disp = result_dir + "disp.cfg" 
	
		self.image.polyphemusDisplay.config = config_disp
		self.image.polyphemusDisplay.map = None
		self.image.polyphemusDisplay.data = None
		self.image.polyphemusDisplay.level = 0
		self.image.polyphemusDisplay.step = 0
		self.image.print_friend()
		self.image.polyphemusDisplay.dispcf()


	def clear(self):
		"""
		Clears the window.
		"""
		self.statusBar().message("Dialog cleared.")
		self.selectPath.clear()
		self.image.clear()
		self.domain.clear()
		self.date.clear()
		self.species.clear()
		self.kz.clear()
		self.meteo.clear()



if __name__ == "__main__":
	app = QApplication(sys.argv)
	f = MainWindow()
	f.show()
	app.setMainWidget(f)
	app.exec_loop()	
