#!/usr/bin/env python

# Copyright (C) 2006-2007 CEREA
#   Author: Vivien Mallet
#
# This code is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This file is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# This module provides facilities to launch Polyphemus programs.


##############
# POLYPHEMUS #
##############


class Polyphemus:
    """
    Class to define a simulation.
    """
    
    def __init__(self, sp):
        """
        Constructor.
        Defines the species involved.

        @type sp: String.
        @param sp: The name of the species considered.
        """
        self.species = sp
        self.program_list = []
        self.log = "-" * 78 + "\n\n"

        
    def AddProgram(self, program):
        """
        Add a program to the list of the simulation.

        @type program: String or Program.
        @param program: The program to add to the list.
        """
        if isinstance(program, str):
            self.program_list.append(Program(program))
        else:
            self.program_list.append(program)


    def Clear(self):
        """
        Clears the simulation: removse the species name, empties the program
        list and cleans the log.
        """
        self.species = ""
        self.program_list = []
        self.log = "-" * 78 + "\n\n"


    def Run(self, program = None):
        """
        Runs a program of the list or all of them.
        """
        if program is None:
            for program in self.program_list:
                print program.basename + "\n"
                program.Run()
                self.log += program.log
                if self.log[-1] != "\n":
                    self.log += "\n"
                self.log += "\n" + "-" * 78 + "\n\n"
                print program.log
                print "\n" + "-" * 78 + "\n\n"
                if program.status != 0:
                    raise Exception, "Program \"" + program.basename \
                          + "\" failed."
        else:
            import os
            basenames = [os.path.basename(x.name) for x in self.program_list]
            i = basenames.index(program)
            program = self.program_list[i]
            program.Run()
            self.log += program.log
            if self.log[-1] != "\n":
                self.log += "\n"
            self.log += "\n" + "-" * 78 + "\n\n"
            print program.log
            print "\n" + "-" * 78 + "\n\n"
            if program.status != 0:
                raise Exception, "Program \"" + program.basename \
                      + "\" failed (status " + str(program.status) + ")."



###########
# PROGRAM #
###########


class Program:
    """
    Class used to define a program used by class Polyphemus.
    """
    def __init__(self, name = None, config = None, format = " %a"):
        if config is not None:
            self.config = config
        else:
            self.config = Configuration()
        self.name = name
        import os
        self.basename = os.path.basename(name)
        self.exec_path = "./"
        self.format = format
        self.priority = "0"
        self.output_directory = None
        self.status = None
        self.log = None


    def Run(self):
        import commands
        if not self.IsReady():
            raise Exception, "Program \"" + self.name + "\" is not ready."
        format = self.format[:]
        command = "nice time " + self.name \
                  + format.replace("%a", self.config.GetArgument())
        self.status, self.log = commands.getstatusoutput(command)


    def SetConfiguration(self, config, mode = None, path = None):
        if isinstance(config, dict):
            self.config.SetConfiguration(config, mode, path)
        elif isinstance(config, list) or isinstance(config, str):
            if mode is None:
                mode = "tmp"
            self.config = Configuration(config, mode, path)
        else:
            self.config = config


    def IsReady(self):
        return self.config.IsReady()
        

    def ClearOutput(self):
        if self.output_directory is not None:
            print "   Output directory cleared."


    def CopyConfiguration(self, directory = None):
        if directory is None:
            directory = self.output_directory
        print "   Configuration copied."



#################
# CONFIGURATION #
#################


class Configuration:
    """
    Class used to define the configuration of a program.
    """

    def __init__(self, file_list = [], mode = "tmp", path = None):
        import os
        if isinstance(file_list, str):
            self.raw_file_list = [file_list]
        else:
            self.raw_file_list = file_list
        for f in self.raw_file_list:
            if not os.path.isfile(f):
                raise Exception, "Unable to find \"" + f + "\"."
        self.file_list = []
        self.ready = False
        self.mode = mode
        if mode not in ["tmp", "path"]:
            raise Exception, "Mode \"" + str(mode) + "\" is not supported."
        if mode == "path" and path is not None:
            self.path = path
        else:
            self.path = "/tmp/"
        self.config = {}


    def IsReady(self):
        return self.ready or self.raw_file_list == []
        

    def SetMode(self, mode, path = None):
        return self.config


    def GetConfiguration(self):
        return self.config


    def SetConfiguration(self, config, mode = None, path = None):
        if mode is not None:
            self.mode = mode
        if path is not None:
            self.path = path
        self.config = config
        self.ProceedReplacement()


    def ProceedReplacement(self):
        import os, shutil, fileinput
        self.file_list = []
        for f in self.raw_file_list:
            if self.mode == "path":
                if os.path.dirname(f) == self.path:
                    raise Exception, "Error: attempt to overwrite" \
                          + " the raw configuration file \"" + f + "\"."
                name = os.path.join(self.path, os.path.basename(f))
                shutil.copy(f, name)
                self.file_list.append(name)
            else: # self.mode == "tmp"
                name = "/tmp/" + os.path.basename(f)
                shutil.copy(f, name)
                self.file_list.append(name)
        for line in fileinput.input(self.file_list, 1):
            new_line = line
            for i in self.config.keys():
                new_line = new_line.replace(str(i), str(self.config[i]))
            print new_line,
        self.ready = True
        

    def GetRawFileList(self):
        return self.raw_file_list


    def SetRawFileList(self, file_list):
        self.raw_file_list = file_list
        self.ready = False


    def Clear(self):
        self.raw_file_list = []
        self.file_list = []
        self.ready = False
        self.mode = "tmp" # or "local" or "path"
        self.path = "./"
        self.config = {}


    def GetArgument(self):
        if self.IsReady():
            return " ".join(self.file_list)
        else:
            raise Exception, "Not ready."


    def SetPath(self, path):
        self.path = path

